;;; spamwash.el

;;; Code:

(require 'mail-parse)
(require 'mail-utils)
(require 'mm-bodies)
(require 'mm-decode)

(defgroup spamwash nil
  "Washing of messages prior to further spam analysis."
  :version "22"
  :group 'spam)

(defcustom spamwash-treatment-alist
  '(("multipart/.*" . spamwash-treat-multipart)
    ("text/.*" . spamwash-decode-body)
    ("image/.*" . spamwash-strip-part)
    ("application/octet-stream" . spamwash-strip-part))
  "An alist of (TYPE . FUNCTION) pairs for `spamwash'.
TYPE is a regexp matching a MIME type and FUNCTION is a function
applied to matching MIME parts."
  :type '(alist :key-type regexp
		:value-type: function)
  :group 'spamwash)

(defun spamwash-decode-body (cte ctl)
  "Decode body according to CTE and charset."
  (let ((charset (or (mail-content-type-get ctl 'charset)
		     "ascii")))
    (set-buffer-multibyte t)
    (mm-decode-body charset cte (car ctl))))

(defun spamwash-strip-part (cte ctl)
  "Remove part."
  (delete-region (point-min) (point-max)))

(defun spamwash-treat-multipart (cte ctl)
  "Treat a multipart MIME part."
  (let ((boundary
	 (concat "^--" (regexp-quote (mail-content-type-get ctl 'boundary))))
	start)
    (while (re-search-forward boundary nil t)
      (forward-line 1)
      (setq start (point))
      (when (re-search-forward boundary nil t)
	(end-of-line 0)
	(save-restriction
	  (narrow-to-region start (point))
	  (spamwash))))))

(defun spamwash ()
  "Wash current buffer."
  (interactive)
  (let (cte ct ctl match buffer-read-only)
    (save-excursion
      (save-restriction
	(mail-narrow-to-head)
	(setq cte (mail-fetch-field "content-transfer-encoding"))
	(setq ctl
	      (if (setq ct (mail-fetch-field "content-type"))
		  (condition-case nil
		      ;; `rfc2231-parse-string' is a major bottleneck,
		      ;; implement something simpler.
		      (rfc2231-parse-string ct)
		    (error '("text/plain")))
		'("text/plain"))))
	(setq match
	      (mm-assoc-string-match spamwash-treatment-alist (car ctl)))
      (when match
	(save-restriction
	  (narrow-to-region
	   (progn
	     (goto-char (point-min))
	     (or (search-forward "\n\n" nil t)
		 (point-max)))
	   (point-max))
	  (funcall (cdr match) cte ctl))))))

(provide 'spamwash)

;;; spamwash.el ends here
